@@ -1,17 +1,18 @@
-
+import os
 import sys
 import datetime
-import pymongo
-
 import base64
 import hmac
 import random
 import hashlib
 
-from alerta.app import app
-from alerta.common.alert import AlertDocument
-from alerta.common.heartbeat import HeartbeatDocument
-from alerta.common import severity_code, status_code
+import pymongo
+
+from alerta.app import app, severity_code, status_code
+
+from alerta.alert import AlertDocument
+from alerta.heartbeat import HeartbeatDocument
+
 
 LOG = app.logger
 
@@ -26,6 +27,11 @@ class Mongo(object):
 
     def connect(self):
 
+        if 'MONGO_PORT' in os.environ and 'tcp://' in os.environ['MONGO_PORT']:  # used by linked Docker containers
+            host, port = os.environ['MONGO_PORT'][6:].split(':')
+            app.config['MONGO_HOST'] = host
+            app.config['MONGO_PORT'] = int(port)
+
         if not app.config['MONGO_REPLSET']:
             try:
                 self.conn = pymongo.MongoClient(app.config['MONGO_HOST'], app.config['MONGO_PORT'])
@@ -594,7 +600,9 @@ class Mongo(object):
         """
         query = {'_id': {'$regex': '^' + id}}
 
-        event = self.db.alerts.find_one(query, fields={"event": 1, "_id": 0})['event']
+        event = self.db.alerts.find_one(query, fields={"event": 1, "_id": 0})
+        if not event:
+            return False
 
         now = datetime.datetime.utcnow()
         update = {
@@ -654,7 +662,7 @@ class Mongo(object):
         """
         response = self.db.alerts.update({'_id': {'$regex': '^' + id}}, {'$addToSet': {"tags": {'$each': tags}}})
 
-        return True if 'ok' in response else False
+        return response.get('updatedExisting', False)
 
     def untag_alert(self, id, tags):
         """
@@ -662,13 +670,13 @@ class Mongo(object):
         """
         response = self.db.alerts.update({'_id': {'$regex': '^' + id}}, {'$pullAll': {"tags": tags}})
 
-        return True if 'ok' in response else False
+        return response.get('updatedExisting', False)
 
     def delete_alert(self, id):
 
         response = self.db.alerts.remove({'_id': {'$regex': '^' + id}})
 
-        return True if 'ok' in response else False
+        return response.get('ok', False) and response.get('n', 0) == 1
 
     def get_counts(self, query=None):
         """
@@ -817,12 +825,50 @@ class Mongo(object):
                                        new=True,
                                        upsert=True
                                        )["value"]
-            return response['_id']
+            return HeartbeatDocument(
+                id=response['_id'],
+                origin=response['origin'],
+                tags=response['tags'],
+                event_type=response['type'],
+                create_time=response['createTime'],
+                timeout=response['timeout'],
+                receive_time=response['receiveTime']
+            )
         else:
             update = update['$set']
             update["_id"] = heartbeat.id
             response = self.db.heartbeats.insert(update)
-            return response
+
+            return HeartbeatDocument(
+                id=response,
+                origin=update['origin'],
+                tags=update['tags'],
+                event_type=update['type'],
+                create_time=update['createTime'],
+                timeout=update['timeout'],
+                receive_time=update['receiveTime']
+            )
+
+    def get_heartbeat(self, id):
+
+        if len(id) == 8:
+            query = {'$or': [{'_id': {'$regex': '^' + id}}, {'lastReceiveId': {'$regex': '^' + id}}]}
+        else:
+            query = {'$or': [{'_id': id}, {'lastReceiveId': id}]}
+
+        response = self.db.heartbeats.find_one(query)
+        if not response:
+            return
+
+        return HeartbeatDocument(
+            id=response['_id'],
+            tags=response['tags'],
+            origin=response['origin'],
+            event_type=response['type'],
+            create_time=response['createTime'],
+            timeout=response['timeout'],
+            receive_time=response['receiveTime']
+        )
 
     def delete_heartbeat(self, id):
 
@@ -854,7 +900,8 @@ class Mongo(object):
     def delete_user(self, user):
 
         response = self.db.users.remove({"user": user})
-        return True if 'ok' in response else False
+
+        return response.get('ok', False) and response.get('n', 0) == 1
 
     def get_metrics(self):
 
@@ -933,7 +980,8 @@ class Mongo(object):
     def delete_key(self, key):
 
         response = self.db.keys.remove({"key": key})
-        return True if 'ok' in response else False
+
+        return response.get('ok', False) and response.get('n', 0) == 1
 
     def is_token_valid(self, token):
 
