@@ -4,15 +4,14 @@ import requests
 
 from collections import defaultdict
 from functools import wraps
-from flask import request, current_app, render_template, redirect, session, abort
+from flask import request, current_app, render_template, redirect, abort
 
-from alerta.app import app, db
+from alerta.app import app, db, status_code
 from alerta.app.switch import Switch
 from alerta.app.utils import parse_fields, crossdomain
-from alerta.app.metrics import Gauge, Counter, Timer
-from alerta.common.alert import Alert
-from alerta.common.heartbeat import Heartbeat
-from alerta.common import status_code, severity_code
+from alerta.app.metrics import Timer
+from alerta.alert import Alert
+from alerta.heartbeat import Heartbeat
 from alerta.plugins import load_plugins
 
 LOG = app.logger
@@ -53,10 +52,7 @@ def jsonify(*args, **kwargs):
 
 def authenticate():
 
-    response = jsonify(status="error", message="authentication required")
-    response.status_code = 401
-
-    return response
+    return jsonify(status="error", message="authentication required"), 401
 
 
 def verify_token(token):
@@ -74,12 +70,13 @@ def verify_token(token):
         return False
 
     if 'audience' in token_info:
-        if token_info['audience'] != app.config['GOOGLE_OAUTH_CLIENT_ID']:
+        if token_info['audience'] != app.config['OAUTH2_CLIENT_ID']:
             LOG.warning('Token supplied was for different web application')
             return False
 
     if 'email' in token_info:
-        if not (token_info['email'].split('@')[1] in app.config['ALLOWED_EMAIL_DOMAINS']
+        if not ('*' in app.config['ALLOWED_EMAIL_DOMAINS']
+                or token_info['email'].split('@')[1] in app.config['ALLOWED_EMAIL_DOMAINS']
                 or db.is_user_valid(token_info['email'])):
             LOG.info('User %s not authorized to access API', token_info['email'])
             return False
@@ -120,6 +117,9 @@ def auth_required(func):
     @wraps(func)
     def decorated(*args, **kwargs):
 
+        if not app.config['AUTH_REQUIRED']:
+            return func(*args, **kwargs)
+
         if 'Authorization' in request.headers:
             auth = request.headers['Authorization']
             LOG.debug(auth)
@@ -177,10 +177,6 @@ def test():
     )
 
 @app.route('/')
-def root():
-    return redirect('/api', code=302)
-
-@app.route('/api', methods=['GET'])
 def index():
 
     rules = []
@@ -190,7 +186,7 @@ def index():
     return render_template('index.html', rules=rules)
 
 
-@app.route('/api/alerts', methods=['OPTIONS', 'GET'])
+@app.route('/alerts', methods=['OPTIONS', 'GET'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
 @auth_required
 @jsonp
@@ -201,7 +197,7 @@ def get_alerts():
         query, sort, _, limit, query_time = parse_fields(request)
     except Exception, e:
         gets_timer.stop_timer(gets_started)
-        return jsonify(status="error", message=str(e))
+        return jsonify(status="error", message=str(e)), 400
 
     fields = dict()
     fields['history'] = {'$slice': app.config['HISTORY_LIMIT']}
@@ -209,7 +205,11 @@ def get_alerts():
     if 'status' not in query:
         query['status'] = {'$ne': "expired"}  # hide expired if status not in query
 
-    alerts = db.get_alerts(query=query, fields=fields, sort=sort, limit=limit)
+    try:
+        alerts = db.get_alerts(query=query, fields=fields, sort=sort, limit=limit)
+    except Exception as e:
+        return jsonify(status="error", message=str(e)), 500
+
     total = db.get_count(query=query)  # because total may be greater than limit
 
     found = 0
@@ -223,6 +223,7 @@ def get_alerts():
 
         for alert in alerts:
             body = alert.get_body()
+            body['href'] = "%s/%s" % (request.base_url.replace('alerts', 'alert'), alert.id)
             found += 1
             severity_count[body['severity']] += 1
             status_count[body['status']] += 1
@@ -257,9 +258,9 @@ def get_alerts():
             statusCounts=status_count,
             lastTime=query_time,
             autoRefresh=Switch.get('auto-refresh-allow').is_on()
-        )
+        ), 404
 
-@app.route('/api/alerts/history', methods=['OPTIONS', 'GET'])
+@app.route('/alerts/history', methods=['OPTIONS', 'GET'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
 @auth_required
 @jsonp
@@ -268,9 +269,15 @@ def get_history():
     try:
         query, _, _, limit, query_time = parse_fields(request)
     except Exception, e:
-        return jsonify(status="error", message=str(e))
+        return jsonify(status="error", message=str(e)), 400
 
-    history = db.get_history(query=query, limit=limit)
+    try:
+        history = db.get_history(query=query, limit=limit)
+    except Exception as e:
+        return jsonify(status="error", message=str(e)), 500
+
+    for alert in history:
+        alert['href'] = "%s/%s" % (request.base_url.replace('alerts/history', 'alert'), alert['id'])
 
     if len(history) > 0:
         return jsonify(
@@ -284,9 +291,9 @@ def get_history():
             message="not found",
             history=[],
             lastTIme=query_time
-        )
+        ), 404
 
-@app.route('/api/alert', methods=['OPTIONS', 'POST'])
+@app.route('/alert', methods=['OPTIONS', 'POST'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
 @auth_required
 @jsonp
@@ -300,7 +307,16 @@ def receive_alert():
         incomingAlert = Alert.parse_alert(request.data)
     except ValueError, e:
         receive_timer.stop_timer(recv_started)
-        return jsonify(status="error", message=str(e))
+        return jsonify(status="error", message=str(e)), 400
+
+    if incomingAlert:
+        for plugin in plugins:
+            try:
+                reject = plugin.pre_receive(incomingAlert)
+                if reject:
+                    return jsonify(status="error", message=reject), 403
+            except Exception as e:
+                LOG.warning('Error while running pre-receive plug-in: %s', e)
 
     try:
         if db.is_duplicate(incomingAlert):
@@ -311,9 +327,9 @@ def receive_alert():
 
             for plugin in plugins:
                 try:
-                    plugin.send(alert)
+                    plugin.post_receive(alert)
                 except Exception as e:
-                    LOG.warning('Error while running plug-in: %s', e)
+                    LOG.warning('Error while running post-receive plug-in: %s', e)
 
         elif db.is_correlated(incomingAlert):
 
@@ -323,9 +339,9 @@ def receive_alert():
 
             for plugin in plugins:
                 try:
-                    plugin.send(alert)
+                    plugin.post_receive(alert)
                 except Exception as e:
-                    LOG.warning('Error while running plug-in: %s', e)
+                    LOG.warning('Error while running post-receive plug-in: %s', e)
 
         else:
             started = create_timer.start_timer()
@@ -334,36 +350,43 @@ def receive_alert():
 
             for plugin in plugins:
                 try:
-                    plugin.send(alert)
+                    plugin.post_receive(alert)
                 except Exception as e:
-                    LOG.warning('Error while running plug-in: %s', e)
+                    LOG.warning('Error while running post-receive plug-in: %s', e)
 
         receive_timer.stop_timer(recv_started)
 
     except Exception, e:
-        return jsonify(status="error", message=str(e))
+        return jsonify(status="error", message=str(e)), 500
 
     if alert:
-        return jsonify(status="ok", id=alert.id)
+        body = alert.get_body()
+        body['href'] = "%s/%s" % (request.base_url, alert.id)
+        return jsonify(status="ok", id=alert.id, alert=body), 201, {'Location': '%s/%s' % (request.base_url, alert.id)}
     else:
-        return jsonify(status="error", message="alert insert or update failed")
+        return jsonify(status="error", message="alert insert or update failed"), 500
 
 
-@app.route('/api/alert/<id>', methods=['OPTIONS', 'GET'])
+@app.route('/alert/<id>', methods=['OPTIONS', 'GET'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
 @auth_required
 @jsonp
 def get_alert(id):
 
-    alert = db.get_alert(id=id)
+    try:
+        alert = db.get_alert(id=id)
+    except Exception as e:
+        return jsonify(status="error", message=str(e)), 500
 
     if alert:
-        return jsonify(status="ok", total=1, alert=alert.get_body())
+        body = alert.get_body()
+        body['href'] = request.base_url
+        return jsonify(status="ok", total=1, alert=body)
     else:
-        return jsonify(status="ok", message="not found", total=0, alert=None)
+        return jsonify(status="ok", message="not found", total=0, alert=None), 404
 
 
-@app.route('/api/alert/<id>/status', methods=['OPTIONS', 'POST'])
+@app.route('/alert/<id>/status', methods=['OPTIONS', 'POST'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
 @auth_required
 @jsonp
@@ -373,26 +396,29 @@ def set_status(id):
     data = request.json
 
     if data and 'status' in data:
-        alert = db.set_status(id=id, status=data['status'], text=data.get('text', ''))
+        try:
+            alert = db.set_status(id=id, status=data['status'], text=data.get('text', ''))
+        except Exception as e:
+            return jsonify(status="error", message=str(e)), 500
     else:
         status_timer.stop_timer(status_started)
-        return jsonify(status="error", message="no data")
+        return jsonify(status="error", message="must supply 'status' as parameter"), 400
 
     if alert:
         for plugin in plugins:
             try:
-                plugin.send(alert)
+                plugin.post_receive(alert)
             except Exception as e:
-                LOG.warning('Error while running plug-in: %s', e)
+                LOG.warning('Error while running post-receive plug-in: %s', e)
 
         status_timer.stop_timer(status_started)
         return jsonify(status="ok")
     else:
         status_timer.stop_timer(status_started)
-        return jsonify(status="error", message="failed to set alert status")
+        return jsonify(status="error", message="not found"), 404
 
 
-@app.route('/api/alert/<id>/tag', methods=['OPTIONS', 'POST'])
+@app.route('/alert/<id>/tag', methods=['OPTIONS', 'POST'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
 @auth_required
 @jsonp
@@ -402,19 +428,22 @@ def tag_alert(id):
     data = request.json
 
     if data and 'tags' in data:
-        response = db.tag_alert(id, data['tags'])
+        try:
+            response = db.tag_alert(id, data['tags'])
+        except Exception as e:
+            return jsonify(status="error", message=str(e)), 500
     else:
         tag_timer.stop_timer(tag_started)
-        return jsonify(status="error", message="no data")
+        return jsonify(status="error", message="must supply 'tags' as list parameter"), 400
 
     tag_timer.stop_timer(tag_started)
     if response:
         return jsonify(status="ok")
     else:
-        return jsonify(status="error", message="failed to tag alert")
+        return jsonify(status="error", message="not found"), 404
 
 
-@app.route('/api/alert/<id>/untag', methods=['OPTIONS', 'POST'])
+@app.route('/alert/<id>/untag', methods=['OPTIONS', 'POST'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
 @auth_required
 @jsonp
@@ -424,19 +453,22 @@ def untag_alert(id):
     data = request.json
 
     if data and 'tags' in data:
-        response = db.untag_alert(id, data['tags'])
+        try:
+            response = db.untag_alert(id, data['tags'])
+        except Exception as e:
+            return jsonify(status="error", message=str(e)), 500
     else:
         untag_timer.stop_timer(untag_started)
-        return jsonify(status="error", message="no data")
+        return jsonify(status="error", message="must supply 'tags' as list parameter"), 400
 
     untag_timer.stop_timer(untag_started)
     if response:
         return jsonify(status="ok")
     else:
-        return jsonify(status="error", message="failed to un-tag alert")
+        return jsonify(status="error", message="not found"), 404
 
 
-@app.route('/api/alert/<id>', methods=['OPTIONS', 'DELETE', 'POST'])
+@app.route('/alert/<id>', methods=['OPTIONS', 'DELETE', 'POST'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
 @auth_required
 @jsonp
@@ -445,17 +477,20 @@ def delete_alert(id):
     if (request.method == 'DELETE' or
             (request.method == 'POST' and '_method' in request.json and request.json['_method'] == 'delete')):
         started = delete_timer.start_timer()
-        response = db.delete_alert(id)
+        try:
+            response = db.delete_alert(id)
+        except Exception as e:
+            return jsonify(status="error", message=str(e)), 500
         delete_timer.stop_timer(started)
 
         if response:
             return jsonify(status="ok")
         else:
-            return jsonify(status="error", message="failed to delete alert")
+            return jsonify(status="error", message="not found"), 404
 
 
 # Return severity and status counts
-@app.route('/api/alerts/count', methods=['OPTIONS', 'GET'])
+@app.route('/alerts/count', methods=['OPTIONS', 'GET'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
 @auth_required
 @jsonp
@@ -463,10 +498,13 @@ def get_counts():
 
     try:
         query, _, _, _, query_time = parse_fields(request)
-    except Exception, e:
-        return jsonify(status="error", message=str(e))
+    except Exception as e:
+        return jsonify(status="error", message=str(e)), 400
 
-    counts = db.get_counts(query=query)
+    try:
+        counts = db.get_counts(query=query)
+    except Exception as e:
+        return jsonify(status="error", message=str(e)), 500
 
     found = 0
     severity_count = defaultdict(int)
@@ -477,18 +515,23 @@ def get_counts():
         severity_count[count['severity']] += 1
         status_count[count['status']] += 1
 
-    return jsonify(
-        status="ok",
-        total=found,
-        more=False,
-        severityCounts=severity_count,
-        statusCounts=status_count,
-        lastTime=query_time,
-        autoRefresh=Switch.get('auto-refresh-allow').is_on()
-    )
+    if found:
+        return jsonify(
+            status="ok",
+            total=found,
+            severityCounts=severity_count,
+            statusCounts=status_count
+        )
+    else:
+        return jsonify(
+            status="ok",
+            message="not found",
+            total=0,
+            severityCounts=severity_count,
+            statusCounts=status_count
+        ), 404
 
-
-@app.route('/api/alerts/top10', methods=['OPTIONS', 'GET'])
+@app.route('/alerts/top10', methods=['OPTIONS', 'GET'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
 @auth_required
 @jsonp
@@ -496,10 +539,17 @@ def get_top10():
 
     try:
         query, _, group, _, query_time = parse_fields(request)
-    except Exception, e:
-        return jsonify(status="error", message=str(e))
+    except Exception as e:
+        return jsonify(status="error", message=str(e)), 400
 
-    top10 = db.get_topn(query=query, group=group, limit=10)
+    try:
+        top10 = db.get_topn(query=query, group=group, limit=10)
+    except Exception as e:
+        return jsonify(status="error", message=str(e)), 500
+
+    for item in top10:
+        for resource in item['resources']:
+            resource['href'] = "%s/%s" % (request.base_url.replace('alerts/top10', 'alert'), resource['id'])
 
     if top10:
         return jsonify(
@@ -513,9 +563,9 @@ def get_top10():
             message="not found",
             total=0,
             top10=[],
-        )
+        ), 404
 
-@app.route('/api/environments', methods=['OPTIONS', 'GET'])
+@app.route('/environments', methods=['OPTIONS', 'GET'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
 @auth_required
 @jsonp
@@ -523,10 +573,13 @@ def get_environments():
 
     try:
         query, _, _, limit, query_time = parse_fields(request)
-    except Exception, e:
-        return jsonify(status="error", message=str(e))
+    except Exception as e:
+        return jsonify(status="error", message=str(e)), 400
 
-    environments = db.get_environments(query=query, limit=limit)
+    try:
+        environments = db.get_environments(query=query, limit=limit)
+    except Exception as e:
+        return jsonify(status="error", message=str(e)), 500
 
     if environments:
         return jsonify(
@@ -540,10 +593,10 @@ def get_environments():
             message="not found",
             total=0,
             environments=[],
-        )
+        ), 404
 
 
-@app.route('/api/services', methods=['OPTIONS', 'GET'])
+@app.route('/services', methods=['OPTIONS', 'GET'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
 @auth_required
 @jsonp
@@ -551,10 +604,13 @@ def get_services():
 
     try:
         query, _, _, limit, query_time = parse_fields(request)
-    except Exception, e:
-        return jsonify(status="error", message=str(e))
+    except Exception as e:
+        return jsonify(status="error", message=str(e)), 400
 
-    services = db.get_services(query=query, limit=limit)
+    try:
+        services = db.get_services(query=query, limit=limit)
+    except Exception, e:
+        return jsonify(status="error", message=str(e)), 500
 
     if services:
         return jsonify(
@@ -568,9 +624,9 @@ def get_services():
             message="not found",
             total=0,
             services=[],
-        )
+        ), 404
 
-@app.route('/api/pagerduty', methods=['OPTIONS', 'POST'])
+@app.route('/pagerduty', methods=['OPTIONS', 'POST'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept'])
 def pagerduty():
 
@@ -638,33 +694,48 @@ def pagerduty():
             alert.origin = 'pagerduty/webhook'
             for plugin in plugins:
                 try:
-                    plugin.send(alert)
+                    plugin.post_receive(alert)
                 except Exception as e:
-                    LOG.warning('Error while running plug-in: %s', e)
+                    LOG.warning('Error while running post-receive plug-in: %s', e)
 
     return jsonify(status="ok")
 
 
 # Return a list of heartbeats
-@app.route('/api/heartbeats', methods=['OPTIONS', 'GET'])
+@app.route('/heartbeats', methods=['OPTIONS', 'GET'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
 @auth_required
 @jsonp
 def get_heartbeats():
 
-    heartbeats = db.get_heartbeats()
+    try:
+        heartbeats = db.get_heartbeats()
+    except Exception as e:
+        return jsonify(status="error", message=str(e)), 500
+
     hb_list = list()
     for hb in heartbeats:
-        hb_list.append(hb.get_body())
+        body = hb.get_body()
+        body['href'] = "%s/%s" % (request.base_url.replace('heartbeats', 'heartbeat'), hb.id)
+        hb_list.append(body)
 
-    return jsonify(
-        status="ok",
-        total=len(heartbeats),
-        heartbeats=hb_list,
-        time=datetime.datetime.utcnow()
-    )
+    if hb_list:
+        return jsonify(
+            status="ok",
+            total=len(heartbeats),
+            heartbeats=hb_list,
+            time=datetime.datetime.utcnow()
+        )
+    else:
+        return jsonify(
+            status="ok",
+            message="not found",
+            total=0,
+            heartbeats=hb_list,
+            time=datetime.datetime.utcnow()
+        ), 404
 
-@app.route('/api/heartbeat', methods=['OPTIONS', 'POST'])
+@app.route('/heartbeat', methods=['OPTIONS', 'POST'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
 @auth_required
 @jsonp
@@ -672,44 +743,83 @@ def create_heartbeat():
 
     try:
         heartbeat = Heartbeat.parse_heartbeat(request.data)
-    except ValueError, e:
-        return jsonify(status="error", message=str(e))
+    except ValueError as e:
+        return jsonify(status="error", message=str(e)), 400
 
-    heartbeat_id = db.save_heartbeat(heartbeat)
+    try:
+        heartbeat = db.save_heartbeat(heartbeat)
+    except Exception as e:
+        return jsonify(status="error", message=str(e)), 500
 
-    return jsonify(status="ok", id=heartbeat_id)
+    body = heartbeat.get_body()
+    body['href'] = "%s/%s" % (request.base_url, heartbeat.id)
+    return jsonify(status="ok", id=heartbeat.id, heartbeat=body), 201, {'Location': '%s/%s' % (request.base_url, heartbeat.id)}
 
-@app.route('/api/heartbeat/<id>', methods=['OPTIONS', 'DELETE', 'POST'])
+@app.route('/heartbeat/<id>', methods=['OPTIONS', 'GET'])
+@crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
+@auth_required
+@jsonp
+def get_heartbeat(id):
+
+    try:
+        heartbeat = db.get_heartbeat(id=id)
+    except Exception as e:
+        return jsonify(status="error", message=str(e)), 500
+
+    if heartbeat:
+        body = heartbeat.get_body()
+        body['href'] = request.base_url
+        return jsonify(status="ok", total=1, heartbeat=body)
+    else:
+        return jsonify(status="ok", message="not found", total=0, heartbeat=None), 404
+
+@app.route('/heartbeat/<id>', methods=['OPTIONS', 'DELETE', 'POST'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
 @auth_required
 @jsonp
 def delete_heartbeat(id):
 
     if request.method == 'DELETE' or (request.method == 'POST' and request.json['_method'] == 'delete'):
-        response = db.delete_heartbeat(id)
+        try:
+            response = db.delete_heartbeat(id)
+        except Exception as e:
+            return jsonify(status="error", message=str(e)), 500
 
         if response:
             return jsonify(status="ok")
         else:
-            return jsonify(status="error", message="failed to delete heartbeat")
+            return jsonify(status="error", message="not found"), 404
 
-@app.route('/api/users', methods=['OPTIONS', 'GET'])
+@app.route('/users', methods=['OPTIONS', 'GET'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
 @auth_required
 @jsonp
 def get_users():
 
-    users = db.get_users()
+    try:
+        users = db.get_users()
+    except Exception as e:
+        return jsonify(status="error", message=str(e)), 500
 
-    return jsonify(
-        status="ok",
-        total=len(users),
-        users=users,
-        domains=app.config['ALLOWED_EMAIL_DOMAINS'],
-        time=datetime.datetime.utcnow()
-    )
+    if len(users):
+        return jsonify(
+            status="ok",
+            total=len(users),
+            users=users,
+            domains=app.config['ALLOWED_EMAIL_DOMAINS'],
+            time=datetime.datetime.utcnow()
+        )
+    else:
+        return jsonify(
+            status="ok",
+            message="not found",
+            total=0,
+            users=[],
+            domains=app.config['ALLOWED_EMAIL_DOMAINS'],
+            time=datetime.datetime.utcnow()
+        ), 404
 
-@app.route('/api/user', methods=['OPTIONS', 'POST'])
+@app.route('/user', methods=['OPTIONS', 'POST'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
 @auth_required
 @jsonp
@@ -722,61 +832,87 @@ def create_user():
             "user": user,
             "sponsor": sponsor
         }
-
-        key = db.save_user(data)
+        try:
+            db.save_user(data)
+        except Exception as e:
+            return jsonify(status="error", message=str(e)), 500
     else:
-        key = None
+        return jsonify(status="error", message="must supply 'user' and 'sponsor' as parameters"), 400
 
-    if key:
-        return jsonify(status="ok")
-    else:
-        return jsonify(status="error", message="failed to create user")
+    return jsonify(status="ok"), 201, {'Location': '%s/%s' % (request.base_url, user)}
 
-@app.route('/api/user/<user>', methods=['OPTIONS', 'DELETE', 'POST'])
+@app.route('/user/<user>', methods=['OPTIONS', 'DELETE', 'POST'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
 @auth_required
 @jsonp
 def delete_user(user):
 
     if request.method == 'DELETE' or (request.method == 'POST' and request.json['_method'] == 'delete'):
-        response = db.delete_user(user)
+        try:
+            response = db.delete_user(user)
+        except Exception as e:
+            return jsonify(status="error", message=str(e)), 500
 
         if response:
             return jsonify(status="ok")
         else:
-            return jsonify(status="error", message="failed to delete user")
+            return jsonify(status="error", message="not found"), 404
 
-@app.route('/api/keys', methods=['OPTIONS', 'GET'])
+@app.route('/keys', methods=['OPTIONS', 'GET'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
 @auth_required
 @jsonp
 def get_keys():
 
-    keys = db.get_keys()
+    try:
+        keys = db.get_keys()
+    except Exception as e:
+        return jsonify(status="error", message=str(e)), 500
 
-    return jsonify(
-        status="ok",
-        total=len(keys),
-        keys=keys,
-        time=datetime.datetime.utcnow()
-    )
+    if len(keys):
+        return jsonify(
+            status="ok",
+            total=len(keys),
+            keys=keys,
+            time=datetime.datetime.utcnow()
+        )
+    else:
+        return jsonify(
+            status="ok",
+            message="not found",
+            total=0,
+            keys=[],
+            time=datetime.datetime.utcnow()
+        ), 404
 
-@app.route('/api/keys/<user>', methods=['OPTIONS', 'GET'])
+@app.route('/keys/<user>', methods=['OPTIONS', 'GET'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
 @auth_required
 @jsonp
 def get_user_keys(user):
 
-    keys = db.get_keys({"user": user})
+    try:
+        keys = db.get_keys({"user": user})
+    except Exception as e:
+        return jsonify(status="error", message=str(e)), 500
 
-    return jsonify(
-        status="ok",
-        total=len(keys),
-        keys=keys,
-        time=datetime.datetime.utcnow()
-    )
+    if len(keys):
+        return jsonify(
+            status="ok",
+            total=len(keys),
+            keys=keys,
+            time=datetime.datetime.utcnow()
+        )
+    else:
+        return jsonify(
+            status="ok",
+            message="not found",
+            total=0,
+            keys=[],
+            time=datetime.datetime.utcnow()
+        ), 404
 
-@app.route('/api/key', methods=['OPTIONS', 'POST'])
+@app.route('/key', methods=['OPTIONS', 'POST'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
 @auth_required
 @jsonp
@@ -788,26 +924,28 @@ def create_key():
             "user": user,
             "text": request.json.get("text", "API Key for %s" % user)
         }
-
-        key = db.create_key(data)
+        try:
+            key = db.create_key(data)
+        except Exception as e:
+            return jsonify(status="error", message=str(e)), 500
     else:
-        key = None
+        return jsonify(status="error", message="must supply 'user' as parameter"), 400
 
-    if key:
-        return jsonify(status="ok", key=key)
-    else:
-        return jsonify(status="error", message="failed to generate api key")
+    return jsonify(status="ok", key=key), 201, {'Location': '%s/%s' % (request.base_url, key)}
 
-@app.route('/api/key/<path:key>', methods=['OPTIONS', 'DELETE', 'POST'])
+@app.route('/key/<path:key>', methods=['OPTIONS', 'DELETE', 'POST'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
 @auth_required
 @jsonp
 def delete_key(key):
 
     if request.method == 'DELETE' or (request.method == 'POST' and request.json['_method'] == 'delete'):
-        response = db.delete_key(key)
+        try:
+            response = db.delete_key(key)
+        except Exception as e:
+            return jsonify(status="error", message=str(e)), 500
 
         if response:
             return jsonify(status="ok")
         else:
-            return jsonify(status="error", message="failed to delete api key")
+            return jsonify(status="error", message="not found"), 404
